//Fait par Andr Boileau (dernire modification : novembre 2007)

#declare epaisseurTraits = 0.03;

#macro uneLumiere(localisation)
light_source
{
  localisation
  color rgb<1,1,1>
}
#end  

#macro unCercle(R,plein)
   #if (plein)
     disc
     {
       <0,0,0>,
       y,
       R
     }
   #else
     torus
     {
       R,
       epaisseurTraits
     }
   #end
#end

#macro couleurFond(rouge,vert,bleu)
  background { rgb<rouge/255,vert/255,bleu/255> }
#end

#macro couleur(rouge,vert,bleu,transparent)
  pigment { rgbt <rouge/255,vert/255,bleu/255,transparent/100> }
#end

#macro unPrisme(P1,P2)
box
{
  P1,P2
}
#end

#macro uneSphere(centre,rayon)
sphere
{
  centre
  rayon
}
#end

#macro unPlan(vecteur,d)
plane
{
  vecteur
  d
}
#end

#macro unCone(P1,rayon1,P2,rayon2,ouvert)
cone
{
  P1,rayon1,
  P2,rayon2
  #if (ouvert)
    open
  #end
}
#end

#macro unCylindre(P1,P2,rayon,ouvert)
object
{
  unCone(P1,rayon,P2,rayon,ouvert)
}
#end

#macro unSegment(P1,P2)
   unCylindre(P1,P2,epaisseurTraits,false)
#end

#macro echelle(Ax,Ay,Az)
   scale <Ax,Ay,Az>
#end

#macro translation(Tx,Ty,Tz)
   translate <Tx,Ty,Tz>
#end

#macro rotation(Rx,Ry,Rz)
  rotate <Rx,Ry,Rz>
#end

#macro intersectionDroitePlan(P1,P2,d,a)
 intersectionDroitePlanAux(P1,P2,<0,d,0>,<-sin(radians(a)),cos(radians(a)),0>)
#end

#macro intersectionDroitePlanAux(P1,P2,Q,N)
 ((P1) - (vdot((P1)-(Q),(N))/vdot((P2)-(P1),(N)))*((P2)-(P1)))
#end

// Macros inutiles, laisses pour compatibilit avec versions antrieures

#macro uneCamera(localisation,pointe)
camera
{
  location localisation
  look_at pointe
}
#end

#macro agrandissement(Ax,Ay,Az)
   scale <Ax,Ay,Az>
#end

#macro translationP(T)
   translate <T.x,T.y,T.z>
#end

// Tortue 3D

#macro initTortue()
	#declare TortuePos = <0,0,0>;
	#declare TortueAvant = <0,0,1>;
	#declare TortueHaut = <0,1,0>;
	#declare Tortuedroite = <1,0,0>;
	#declare TortueCrayon = true;
	#declare TortueEpais = epaisseurTraits;
	#declare TortueCouleur = <0,0,0,0>;
	#declare TortueRemplis = false;
	#declare TortueCouleurRemplis = <1,1,0,0>;
	#declare TortuePointAncrage = <0,0,0>;
	tangue(90)
#end

#macro avance(d)
   #local Temp = TortuePos + (d)*TortueAvant;
   segment(TortuePos, Temp)
   #declare TortuePos = Temp;
#end

#macro droite(a)
	#local Temp = <0,0,0>;
	#local an = radians(-(a));
	#declare Temp = cos(an)*Tortuedroite + sin(an)*TortueAvant;
	#declare TortueAvant = -sin(an)*Tortuedroite + cos(an)*TortueAvant;
	#declare Tortuedroite = Temp;
#end
#macro tangue(a)
	#local Temp = <0,0,0>;
	#local an = radians(a);
	#declare Temp = cos(an)*TortueAvant + sin(an)*TortueHaut;
	#declare TortueHaut = -sin(an)*TortueAvant + cos(an)*TortueHaut;
	#declare TortueAvant = Temp;
#end
#macro roule(a)
	#local Temp = <0,0,0>;
	#local an = radians(-(a));
	#declare Temp = cos(an)*Tortuedroite + sin(an)*TortueHaut;
	#declare TortueHaut = -sin(an)*Tortuedroite + cos(an)*TortueHaut;
	#declare Tortuedroite = Temp;
#end

#macro fPos(P)
	segment(TortuePos, P)
	#declare TortuePos = P;
#end

#macro fCap(P)
 #if (vlength((P)-TortuePos)=0)
 #else
	#local Temp = <(P).x,TortuePos.y,(P).z>-TortuePos;
	#if (vlength(Temp)=0)
		#declare TortueAvant = vnormalize((P)-TortuePos);
		#declare Tortuedroite = <1,0,0>;
		#declare TortueHaut = vcross(Tortuedroite,TortueAvant);
	#else
		#declare Temp = vnormalize(Temp);
		#declare TortueHaut = <0,1,0>;
		#declare Tortuedroite = vcross(Temp,TortueHaut);
		#declare TortueAvant = vnormalize((P)-TortuePos);
		#declare TortueHaut = vcross(Tortuedroite,TortueAvant);
	#end
 #end
#end

#macro TC(taille) //TailleCrayon
	#declare TortueEpais = (taille);
#end

#macro CC(r,g,b,trans)   //CouleurCrayon
	#declare TortueCouleur = <(r)/255,(g)/255,(b)/255,(trans)/100>;
#end

#macro CR(r,g,b,trans)   //CouleurRemplissage
	#declare TortueCouleurRemplis = <(r)/255,(g)/255,(b)/255,(trans)/100>;
#end

#macro BC()   //BaisseCrayon
	#declare TortueCrayon = true;
#end

#macro LC()   //LeveCrayon
	#declare TortueCrayon = false;
#end

#macro debutRemplir()
	#declare TortueRemplis = true;
	#declare TortuePointAncrage = TortuePos;
#end

#macro finRemplir()
	#declare TortueRemplis = false;
#end

#macro segment (P1,P2)
	#if (TortueCrayon)
		object { unCylindre(P1,P2,TortueEpais,false) 
				 texture {pigment {rgbt TortueCouleur }} }
	#end
	#local V = vcross(P1-TortuePointAncrage,P2-TortuePointAncrage);  // dgnr ?
	#if ( (TortueRemplis) & (vdot(V,V) != 0) )
		object { triangle {TortuePointAncrage, P1, P2} 
				 texture {pigment {rgbt TortueCouleurRemplis }} }
	#end
#end


initTortue()

